@@ -121,6 +121,22 @@
 #include <windows.h>
 extern HMODULE _priv_gst_dll_handle;
 #endif
+#ifdef HAVE_DLFCN_H
+#include <dlfcn.h>
+#endif
+
+/* Use a toolchain-dependent suffix on Windows */
+#ifdef G_OS_WIN32
+# ifdef _MSC_VER
+#  define GST_REGISTRY_FILE_SUFFIX TARGET_CPU "-msvc"
+# else
+#  define GST_REGISTRY_FILE_SUFFIX TARGET_CPU "-mingw"
+# endif
+#else
+# define GST_REGISTRY_FILE_SUFFIX TARGET_CPU
+#endif
+#define GST_REGISTRY_FILE_NAME "registry." GST_REGISTRY_FILE_SUFFIX ".bin"
+
 
 #define GST_CAT_DEFAULT GST_CAT_REGISTRY
 
@@ -380,8 +396,8 @@ was_added:
  *
  * Get the list of paths for the given registry.
  *
- * Returns: (transfer container) (element-type char*): A #GList of paths as
- *     strings. g_list_free after use.
+ * Returns: (transfer container) (element-type filename): A #GList of
+ *     paths as strings. g_list_free after use.
  *
  * MT safe.
  */
@@ -1005,7 +1021,7 @@ gst_registry_lookup_feature_locked (GstRegistry * registry, const char *name)
  *
  * Find a #GstPluginFeature with @name in @registry.
  *
- * Returns: (transfer full): a #GstPluginFeature with its refcount incremented,
+ * Returns: (transfer full) (nullable): a #GstPluginFeature with its refcount incremented,
  *     use gst_object_unref() after usage.
  *
  * MT safe.
@@ -1530,6 +1546,39 @@ load_plugin_func (gpointer data, gpointer user_data)
   }
 }
 
+char *
+priv_gst_get_relocated_libgstreamer (void)
+{
+  char *dir = NULL;
+
+#ifdef G_OS_WIN32
+  {
+    char *base_dir;
+
+    base_dir =
+        g_win32_get_package_installation_directory_of_module
+        (_priv_gst_dll_handle);
+
+    dir = g_build_filename (base_dir, GST_PLUGIN_SUBDIR, NULL);
+    GST_DEBUG ("using DLL dir %s", dir);
+
+    g_free (base_dir);
+  }
+#elif defined(__APPLE__) && defined(HAVE_DLADDR)
+  {
+    Dl_info info;
+
+    if (dladdr (&gst_init, &info)) {
+      dir = g_path_get_dirname (info.dli_fname);
+    } else {
+      return NULL;
+    }
+  }
+#endif
+
+  return dir;
+}
+
 #ifndef GST_DISABLE_REGISTRY
 /* Unref all plugins marked 'cached', to clear old plugins that no
  * longer exist. Returns %TRUE if any plugins were removed */
@@ -1641,7 +1690,7 @@ scan_and_update_registry (GstRegistry * default_registry,
   if (plugin_path == NULL)
     plugin_path = g_getenv ("GST_PLUGIN_SYSTEM_PATH");
   if (plugin_path == NULL) {
-    char *home_plugins;
+    char *home_plugins, *relocated_libgstreamer, *system_plugindir;
 
     GST_DEBUG ("GST_PLUGIN_SYSTEM_PATH not set");
 
@@ -1656,28 +1705,21 @@ scan_and_update_registry (GstRegistry * default_registry,
 
     /* add the main (installed) library path */
 
-#ifdef G_OS_WIN32
-    {
-      char *base_dir;
-      char *dir;
-
-      base_dir =
-          g_win32_get_package_installation_directory_of_module
-          (_priv_gst_dll_handle);
-
-      dir = g_build_filename (base_dir, GST_PLUGIN_SUBDIR,
+    relocated_libgstreamer = priv_gst_get_relocated_libgstreamer ();
+    if (relocated_libgstreamer) {
+      GST_DEBUG ("found libgstreamer-" GST_API_VERSION " library "
+          "at %s", relocated_libgstreamer);
+      system_plugindir = g_build_filename (relocated_libgstreamer,
           "gstreamer-" GST_API_VERSION, NULL);
-      GST_DEBUG ("scanning DLL dir %s", dir);
-
-      changed |= gst_registry_scan_path_internal (&context, dir);
-
-      g_free (dir);
-      g_free (base_dir);
+    } else {
+      system_plugindir = g_strdup (PLUGINDIR);
     }
-#else
-    GST_DEBUG ("scanning main plugins %s", PLUGINDIR);
-    changed |= gst_registry_scan_path_internal (&context, PLUGINDIR);
-#endif
+
+    GST_DEBUG ("using plugin dir %s", system_plugindir);
+    changed |= gst_registry_scan_path_internal (&context, system_plugindir);
+
+    g_clear_pointer (&system_plugindir, g_free);
+    g_clear_pointer (&relocated_libgstreamer, g_free);
   } else {
     gchar **list;
     gint i;
@@ -1735,7 +1777,7 @@ ensure_current_registry (GError ** error)
     registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
   if (registry_file == NULL) {
     registry_file = g_build_filename (g_get_user_cache_dir (),
-        "gstreamer-" GST_API_VERSION, "registry." TARGET_CPU ".bin", NULL);
+        "gstreamer-" GST_API_VERSION, GST_REGISTRY_FILE_NAME, NULL);
   }
 
   if (!_gst_disable_registry_cache) {
